<?php
# Copyright (C) 2011 Matthew Wronka <murder@projects.matt.wronka.org>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, version 3 of the
# License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>
require_once('php/murder/renderer.inc'); 
require_once('php/murder/renderer_component.inc'); 
require_once('php/murder/renderer_default.inc'); 

class Page
{
	static private $instance = false;

	private $user = false;
	private $renderer = false;

	#### page state ###################
	private $maxCache    = true;
	private $lastModified= false;
	private $vary	     = 'Accept';
	private $stylesheets = array();
	private $javascript  = array();
	private $keywords    = array();
	private $title       = false;
	private $statusCode  = 200;
	private $statusMessage= '';
	private $contentType = 'application/xhtml+xml';
	private $pageMessages= array();
	####################################

	private function __construct()
	{
		$this->user = safeGet($_GET,'_user');
		if($this->user && isset($_GET['_R']))
		{
			redirect($_SERVER['REQUEST_URI'] .'/');
			die;
		}
		$arguments = safeGet($_GET,'_arg');
		$sub       = safeGet($_GET,'_sub');
		if($arguments)
		{
			$arguments = explodea('/', $arguments);
		}
		if($stylesheet = config('STYLE')) $this->stylesheets[] = $stylesheet;
		$renderer = false;
		if($sub)
		{
			$handler = PluginStore::getInstance()->getURLHandler($sub, $this->acceptableContentTypes());
			$renderer = safeGet($handler, 'handler');
			if($renderer)
			{
				$ctype = safeGet($handler, 'type');
				$this->renderer = new $renderer($this, $arguments, $ctype);
			}
		}
		else
		{
			$handler = PluginStore::getInstance()->getURLHandler('/', $this->acceptableContentTypes());
			$renderer = safeGet($handler, 'handler');
			if($renderer)
			{
				$ctype = safeGet($handler, 'type');
				// maybe there's a better content type
				$this->renderer = new $renderer($this, $arguments, $ctype);
			}
			if(!$this->renderer)
			{
				$this->renderer = new DefaultRenderer($this, $arguments);
			}
		}
		if(!$this->renderer)
		{
			$handler = PluginStore::getInstance()->getURLHandler($sub, "*/*");
			if($handler)
			{
				$this->statusCode = 406;
			}
			else
			{
				$this->statusCode = 404;
			}
			$this->renderer = new PageRenderer($this, false);
			if(!$this->renderer) die("Shoot.");
		}
	}

	static public function getInstance()
	{
		if(!Page::$instance)
		{
			Page::$instance = new Page();
		}
		return Page::$instance;
	}

	#############################################
	## Rendering Methods
	#############################################
	public function render()
	{
		$r = $this->renderer;
		if(!$r) return;
		session_start();
		$r->processRequest($this);
		$r->sendHeaders($this);
		if($this->shouldRenderPage())
		{
			$r->startPage($this);
			$r->mainContent($this);
			$r->endPage($this);
		}
	}

	private function shouldRenderPage()
	{
		switch($this->statusCode)
		{
			case 304:
			case 406:
			case 429:
				return false;
		}
		return true;
	}

	public function acceptableContentTypes()
	{
		return safeGet($_SERVER,'HTTP_ACCEPT');
	}

	public function setLastModified($time)
	{
		if(!$this->lastModified || $time > $this->lastModified)
		{
			$this->lastModified = $time;
		}
	}
	public function getLastModified($format=false) 
	{
		if($format)
		{
			if(!$this->lastModified) return false;
			return date('r',$this->lastModified);
		} else {
			return $this->lastModified;
		}
	}

	public function setTitle($title) { $this->title = $title; }

	public function getTitle() { 
		if($this->title) return $this->title; 
		return config('DEFAULT_PAGE_TITLE');
	}

	public function setContentType($type)
	{
		$this->contentType = $type;
	}
	public function getContentType() 
	{
		return $this->contentType;
	}

	public function setStatus($code, $message=false)
	{
		$this->statusCode = $code;
		$this->statusMessage= $message;
	}

	public function getStatusCode() { return $this->statusCode; }
	public function getStatusMessage()
	{
		if($this->statusMessage) return $this->statusMessage;
		switch($this->statusCode)
		{
			case 200: return 'Ok';
			case 300: return 'Multiple Choices';
			case 304: return 'Not Modified';
			case 400: return 'Bad Request';
			case 401: return 'Unauthorized';
			case 402: return 'Payment Required';
			case 403: return 'Forbidden';
			case 404: return 'Not Found';
			case 406: return 'None Acceptable';
			case 410: return 'Gone';
			case 415: return 'Unsupported Media Type';
			case 418: return 'I\'m a Teapot';
			case 429: return 'Too many requests.';
			case 500: return 'Internal Error';
			case 503: return 'Service Unavailable';
			default: return 'Unknknown.';
		}
	}

	public function setMaxAge($age) { $this->maxCache = $age; }
	public function getMaxAge() 
	{ 
		if($this->maxCache === true) return config('HTTP_CACHE_MAX_AGE');
		return $this->maxCache; 
	}
	public function setVary($vary) { $this->vary = $vary; }
	public function getVary() { return $this->vary; }

	public function getStylesheets() { return $this->stylesheets; }

	public function msg($t=false) { if($t) $this->pageMessages[] = $t; else return $this->pageMessages; }

	protected function getComponent($component)
	{
		$comp = PluginStore::getInstance()->getPageComponent($component);
		if(!$comp) return false;
		$renderer = safeGet($comp, 'renderer', false);
		if(!$renderer) return false;
		$rndr = new $renderer($this, false, $this->getContentType());
		if(!$rndr) return false;
		return $rndr;
	}

	public function prepareComponent($component)
	{
		$rndr = $this->getComponent($component);
		if(!$rndr) return false;
	}

	public function renderComponent($component)
	{
		$rndr = $this->getComponent($component);
		if(!$rndr) return false;
		$defer = safeGet($comp, 'defer', false);
		// TODO defer
		$rndr->startComponent($component);
		$rndr->componentContent($this, $component);
		$rndr->endComponent($component);
	}

	#############################################
	## Common Methods
	#############################################
	public function getUser()
	{
		return UserStore::getInstance()->getUser($this->user);
	}

	public function myUrl()
	{
		return baseUrl() . $_SERVER['REQUEST_URI'];
	}
}
